// gcc ivorbisfile_example.c -o ivorbisfile_example -lvorbisidec
// ivorbisfile_example < a.ogg | aplay -t raw -c 2 -f S16_BE -r 44100 -

/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis 'TREMOR' CODEC SOURCE CODE.   *
 *                                                                  *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis 'TREMOR' SOURCE CODE IS (C) COPYRIGHT 1994-2002    *
 * BY THE Xiph.Org FOUNDATION http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <tremor/ivorbiscodec.h>
#include <tremor/ivorbisfile.h>
#include <assert.h>

char pcmout[4096]; /* take 4k out of the data segment, not the stack */
#ifdef INSANE_BUFFER
char stdout_buf[1024*512];
char stdin_buf[1024*128];
#endif

int main(int argc, char *argv[]) {
  OggVorbis_File vf;
  int current_section;
  int ret;

  ret = ov_open(stdin, &vf, NULL, 0);
  assert(ret >= 0);

  {
      vorbis_info *vi = ov_info(&vf,-1);
      {
          char **ptr = ov_comment(&vf,-1)->user_comments;
          while (*ptr) {
              fprintf(stderr, "%s\n",*ptr);
              ++ptr;
          }
      }
      fprintf(stderr, "\nBitstream is %d channel, %ldHz\n",
              vi->channels,
              vi->rate);
      fprintf(stderr, "\nDecoded length: %ld samples\n",
              (long)ov_pcm_total(&vf,-1));
      fprintf(stderr, "Encoded by: %s\n\n",
              ov_comment(&vf,-1)->vendor);
#ifdef INSANE_BUFFER
      setbuffer(stdin, stdin_buf, sizeof stdin_buf);
      setbuffer(stdout, stdout_buf, sizeof stdout_buf);
#endif
  }

  for (;;) {
      long retval;
      retval = ov_read(&vf, pcmout, sizeof(pcmout), &current_section);
      if (retval == 0) {
          /* EOF */
          break;
      } else if (retval < 0) {
          /* error in the stream.  Not a problem, just reporting it in
             case we (the app) cares.  In this case, we don't. */
          assert(0);
      } else {
          /* we don't bother dealing with sample rate changes, etc, but
             you'll have to*/
          fwrite(pcmout, ret, 1, stdout);
      }
  }
  
  /* cleanup */
  ov_clear(&vf);
    
  fprintf(stderr, "Done.\n");
  return(0);
}
